'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.redo = exports.undo = undefined;

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.chain = chain;
exports.watchHistory = watchHistory;

var _mobx = require('mobx');

var _util = require('./util');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Block off change monitoring while undoing and redoing
var isUndoing = false;
var isRedoing = false;

// Actions is an object that holds a stack of batched changes for each action
var actions = {};

// Chain multiple pure functions together to allow declarative querying
function chain(data, funcs) {
  return _util.flow.apply(undefined, (0, _toConsumableArray3.default)((0, _util.concat)([], funcs)))(Array.isArray(data) ? data : (0, _mobx.toJS)(data));
}

// Undo an action
var undo = exports.undo = (0, _mobx.action)('Undo', function undo(actionName) {
  isUndoing = true;
  actions[actionName].future.unshift(actions[actionName].past.pop().map(_util.revertChange).reverse());
  isUndoing = false;
});

// Redo an action
var redo = exports.redo = (0, _mobx.action)('Redo', function redo(actionName) {
  isRedoing = true;
  actions[actionName].past.unshift(actions[actionName].future.pop().map(_util.revertChange).reverse());
  isRedoing = false;
});

// Initialize change spying for undo/redo
function watchHistory() {
  var depth = 0;
  var currentDepth = 0;
  var currentAction = '';

  return (0, _mobx.spy)(function (change) {
    if (isUndoing || isRedoing) return;
    if (change.spyReportEnd) depth--;
    if (change.spyReportStart) depth++;

    // Ignore nested actions
    if (!currentAction && change.type === 'action') {
      // Save current action
      currentDepth = depth;
      currentAction = change.name;

      // Add it to the past
      if (!actions[currentAction]) actions[currentAction] = { past: [], future: [] };
      actions[currentAction].past.unshift([]);
    } else if (currentAction && depth >= currentDepth) {
      // Everything with > depth is part of the current action
      if (change.type) actions[currentAction].past[0].unshift(change);
    } else {
      // Reset
      currentDepth = 0;
      currentAction = '';
    }
  });
}